/*
 * Access a filesystem.
 *
 * Copyright (C) 2018, 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include <fsclient/client.h>

static void read_data(file_descriptor_t *desc)
{
  char buffer[80];
  ssize_t available, limit = 80;

  available = client_read(desc, buffer, limit);

  if (!available)
  {
    printf("\nStart again...\n");
    client_seek(desc, 0, SEEK_SET);
    return;
  }

  /* Consume some of the data. */

  fputs("\n", stdout);
  fwrite((void *) buffer, sizeof(char), available, stdout);
  fflush(stdout);
}

int main(void)
{
  int i;

  /* Re-mount a filesystem. */

  printf("Re-mounting...\n");

  client_mount("etc", "messages/");

  /* Obtain a reference to a file. */

  printf("Open file...\n");

  file_descriptor_t *desc1 = client_open("usr/share/common-licenses/GPL-3", 0);
  file_descriptor_t *desc2 = client_open("usr/share/common-licenses/GPL-2", 0);
  /* file_descriptor_t *desc2 = client_open("messages/motd", 0); */

  if (desc1 == NULL)
  {
    printf("Could not open file #1.\n");
    while (1) sleep(1);
  }

  if (desc2 == NULL)
  {
    printf("Could not open file #2.\n");
    while (1) sleep(1);
  }

  /* Request data from the objects. */

  printf("Reading...\n");

  i = 3;

  while (i--)
  {
    read_data(desc1);
    read_data(desc2);
    sleep(1);
  }

  /* Close files explicitly, but not doing so will cause their closure, anyway. */

  printf("\nClosing...\n");

  client_close(desc1);
  client_close(desc2);

  return 0;
}
